# .*. coding: utf-8 .*.
#---------------------- Copyright (c) ABB Bomem inc, 2012 -------------------
#                               Source code
# This software is the property of Bomem and should be considered and treated
# as proprietary information.  Refer to the "Source Code License Agreement"
#----------------------------------------------------------------------------

import re
import time
import serial

from MyError import MyError

WAITPROMPT_TIMEOUT_SEC = 10


class Console(object):
    """ Class to control a console or cli through a serial port
    """
    ser = None
    def __init__(self):
        self.dump_file = None

    def open(self, com=None, baudrate=115200) :
        """ Open the serial port
        """
        try:
            self.ser = serial.Serial(com)
            self.ser.baudrate = baudrate
            self.ser.timeout = 0.04

        except (serial.serialutil.SerialException, OSError):
            self.close()
            raise MyError('Serial port can\'t be open (' + com + ')')

        return True

    def dump_console(self, filename):
        """ Dump data receive from console to a file
        """
        self.dump_file = open(filename, 'a')

    def check_prompt(self):
        """ Wait for the prompt from device
        """
        success = False
        self.ser.write('\n')

        out = ''.join(self.ser.readlines())
        if self.dump_file is not None:
            self.dump_file.write(out)

        result = re.search('^root@.*:.*#', out, re.MULTILINE | re.IGNORECASE)
        if result:
            success = True

        return success

    def send_command(self, command):
        """ Send a command
        """
        try:  # Try to reopen
            self.ser.write(command + '\n')
        except:
            self.close()
            self.open()

        r = self.wait_for_prompt()
        return r

    def wait_for_prompt(self, prompt='^root@.*:.*#', timeout=WAITPROMPT_TIMEOUT_SEC):
        """ Wait for prompt
        """
        currentTime = firstTime = time.time()
        resp = ''
        while True:
            resp_line = self.ser.readline()
            if self.dump_file is not None:
                self.dump_file.write(resp_line)
                self.dump_file.flush()
            resp = resp + resp_line
            # Find prompt
            result = re.search(prompt, resp, re.MULTILINE | re.IGNORECASE)
            if result:
                break

            currentTime = time.time()

            if (currentTime - firstTime) >= timeout:
                raise MyError('Console didn\'t respond or received an incorrect prompt (' + str(currentTime - firstTime) + ' sec)')

        return resp

    def readlines(self):
        """ Return lines into a string instead of an array
        """
        return ''.join(self.ser.readlines())

    def close(self):
        """ Free resource of serial port
        """
        if self.ser:
            self.ser.close()
        self.ser = None
