#!/usr/bin/python
# .*. coding: utf-8 .*.
""" this file update h2scan firmware for a Coresense1
"""

#---------------------- Copyright (c) ABB Bomem inc, 2012 -------------------
#                               Source code
# This software is the property of Bomem and should be considered and treated
# as proprietary information.  Refer to the "Source Code License Agreement"
#----------------------------------------------------------------------------

# pylint: disable=invalid-name

import sys
import os
import time
#import traceback
import re

from H2Scan import H2Scan

FACTORY_PROPERTIES = 'factory.properties'


def readSerialNumber():
    """ Read serial number in factory file
    """
    fsn = open('/media/eeprom0/' + FACTORY_PROPERTIES, 'r')
    test = fsn.read()
    fsn.close()
    serial = 'None'
    model = 'None'
    result = re.search('factory.serial-number *= *string:(.*?)$', test, re.MULTILINE | re.IGNORECASE)
    if result:
        serial = result.group(1).strip('\r')

    result = re.search('factory.model-number *= *string:(.*?)$', test, re.MULTILINE | re.IGNORECASE)
    if result:
        model = result.group(1).strip('\r')

    return serial, model


class H2LedControl(object):
    """ This object must containt led_on est led_off
    """

    def led_on(self):
        """ Turn a led on to show flashing progression
        """
        os.popen('TransfoCarrier -lh2=b')

    def led_off(self):
        """ Turn a led off to show flashing progression
        """
        os.popen('TransfoCarrier -lh2=o')

# MAIN Program
print 'UpdateH2ScanFirmware\n'
print 'All ABB software must be stopped prior to update.'
print 'Firmware File must be present in the current directory with the CRC File: firmxxx.hex firmxxx.crc'
print 'FirmwareFile @ 0 will only put the Sensor board in client mode to get a std microcom session.\n\n'

try:
    # Check input arg...
    #print sys.argv, '\n'
    if len(sys.argv) != 2:
        print '\tUSAGE: updateH2ScanFirmware FirmwareFile|0 \n\n'
        sys.exit()

    arg = sys.argv

    COMPORT = '/dev/ttymxc4'

    firm_filename = arg[1]

    os.popen('TransfoCarrier -lsys=y -lh2o=o -lh2=b')

    h2scan = H2Scan(COMPORT, H2LedControl())

    # if firmfile = 0 we will just quit the modbus
    if firm_filename != '0':
        CHKSUM = h2scan.read_firmware_crc(firm_filename)

    h2scan.set_h2scan_in_cli_mode()

    if firm_filename == '0':
        sys.exit(-1)

    print 'Open COM port... ',
    try:
        h2scan.open_cli()
        print 'Ok'
    except:
        print 'Error: Opening COM Port.\n'
        sys.exit(-1)

    # Setup a log file
    dev_serial, dev_model = readSerialNumber()
    timestr = time.strftime('%Y%m%d_%H%M%S', time.localtime())
    filename = 'update_' + dev_serial + '_' + dev_model + '_' + timestr + '.log'
    h2scan.dump_console(filename)

    # Ask prompt of H2Scan
    h2scan.ask_h2scan_prompt()

    # Set Advanced mode in H2Scan
    h2scan.set_h2scan_advanced()

    print 'Current H2Scan firmware version: %s' % (h2scan.get_h2scan_version())

    success = h2scan.set_h2scan_updateFirmware(CHKSUM, firm_filename)

    h2scan.close_cli()

    if success is True:
        os.popen('TransfoCarrier -lh2o=o -lh2=cg')
    else:
        os.popen('TransfoCarrier -lh2o=o -lh2=cr')
except:
#    trc = traceback.format_exc().split('\n')
#    for linedebug in trc:
#        print linedebug
    os.popen('TransfoCarrier -lh2o=o -lh2=cr')
